SystemOrganization addCategory: #'Moe-Model'!
SystemOrganization addCategory: #'Moe-Model-Tests'!

TestCase subclass: #MOModelTest
	instanceVariableNames: 'setUpClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Moe-Model-Tests'!

!MOModelTest methodsFor: 'private' stamp: 'jmck 7/20/2013 00:47'!
attributesFor: aClass
	^(self relationFor: aClass) discoverAttributes! !

!MOModelTest methodsFor: 'private' stamp: 'jmck 7/20/2013 00:46'!
relationFor: aClass
	^MAConnectionPool instance relationNamed: aClass tableName! !

!MOModelTest methodsFor: 'setup' stamp: 'jmck 7/20/2013 00:27'!
setUp
	self setUpClass ifNil: [ ^self error: 'No setup class found' ].
	setUpClass setUpDatabase.! !

!MOModelTest methodsFor: 'setup' stamp: 'jmck 7/20/2013 00:52'!
setUpClass
	^setUpClass ifNil: [ setUpClass := self allSubclasses
		detect: [ :cls | cls respondsTo: #setUpDatabase ] ifNone: [nil]]! !

!MOModelTest methodsFor: 'setup' stamp: 'jmck 7/20/2013 00:29'!
startUpClasses
	^setUpClass startUpClasses! !

!MOModelTest methodsFor: 'running' stamp: 'jmck 7/20/2013 00:45'!
testAttributesMatchDescription
	self startUpClasses ifNotNil: [ :classlist |
		classlist do: [ :cls |
			| map rslt |
			map := cls mappedDescription.
			(self attributesFor: cls) do: [ :attr |
				rslt := map
					detect: [ :desc | desc sqlColumnName = attr name ]
					ifNone: [ nil ].
			self assert: rslt notNil ]]]! !

!MOModelTest methodsFor: 'running' stamp: 'jmck 7/20/2013 00:51'!
testDescriptionMatchesAttributes
	self startUpClasses ifNotNil: [ :classlist |
		classlist do: [ :cls |
			| attribs rslt |
			attribs := self attributesFor: cls.
			cls mappedDescription do: [ :desc |
				rslt := attribs
					detect: [ :attr | attr name = desc sqlColumnName ]
					ifNone: [ nil ].
			self assert: rslt notNil ]]]! !

MAMappedObject subclass: #MOObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Moe-Model'!

!MOObject commentStamp: 'jmck 6/12/2009 11:10' prior: 0!
Moe is a simple object persistency strategy using Magritte and Roe - Moe!!
The main architectural feature is that Moe make the database "mirror" the Smalltalk objects rather than force the objects to adhere to all sorts of strangulations like pesky foreign key definitions and the like. 

The base class, MOEmbeddedObject,  is a descendant of MAMappedObject. All persisting objects must descend from MOEmbeddedObject and override loadEmbeddedObjectFor: aDescription withId: anId. MODatalistObject and MODatabaseObject are two classes from which you can descend "out of the box". Datalists keep a list of ALL of its instances in a class variable. The latter loads only the objects as they are referenced. 


Describe an embedded object 'Customer' like so:
descriptionCustomer
	^MOEmbeddedObjectDescription new
		objectClass: Customer; 
		embeddedAccessor: #customer;
		label: 'Customer';
		sqlColumnName: 'customer';
		sqlColumnType: 'integer';
		priority: 10;
		yourself
		
When a Moe described objects will have their ID stored in the database in the column where the object should be, and on retrieval from the DB it will be restored with the Smalltalk object. Collections will have an array of integers stored in the DB from which the collection will be restored upon retrieval.

This is pretty lightweight (I think), and is meant only as a means of persistency (read: data backup and restore)
!

MOObject subclass: #MODatabaseObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Moe-Model'!

!MODatabaseObject commentStamp: 'jmck 6/11/2009 18:00' prior: 0!
I am like the Datalist object but I don't load all of the insances of me from the database.!

!MODatabaseObject methodsFor: 'persistence' stamp: 'jmck 9/24/2009 07:22'!
loadEmbeddedObjectFor: aDescription withId: anId

	 ^ aDescription classes first loadId: anId! !

MOObject subclass: #MODatalistObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Moe-Model'!

MODatalistObject class
	instanceVariableNames: 'List ListMutex'!

!MODatalistObject commentStamp: 'jmck 6/11/2009 17:58' prior: 0!
I am really an example class of one way to maintain the database of objects in the image. I load all the instances of me in a List variable!

MODatalistObject class
	instanceVariableNames: 'List ListMutex'!

!MODatalistObject class methodsFor: 'persistency' stamp: 'jmck 2/26/2011 23:12'!
initialize
	self allSubclasses do: [ :each |
		each isRoot
			ifTrue: [ Smalltalk
				addToStartUpList: each;
				addToShutDownList: each ] ]! !

!MODatalistObject class methodsFor: 'persistency' stamp: 'jmck 8/2/2011 20:23'!
isConcrete
	^ [ self tableName notNil ] on: Error do: [ ^false ]! !

!MODatalistObject class methodsFor: 'persistency' stamp: 'jmck 2/26/2011 22:58'!
isRoot
	^false! !

!MODatalistObject class methodsFor: 'persistency' stamp: 'jmck 5/15/2012 18:06'!
list
	[ self tableName ifNil: [ ^nil ] ] on: Error do: [ ^nil ].
	self listMutex critical: [
		List ifNil: [ 
			[
				List := self loadAll asOrderedCollection.
			] 
			on: Error do: [ :ex | Transcript cr; show: ex asString. List := nil ] ] ].
	^List! !

!MODatalistObject class methodsFor: 'persistency' stamp: 'jmck 2/15/2010 12:28'!
listMutex
	^ListMutex ifNil: [ ListMutex := Mutex new ].! !

!MODatalistObject class methodsFor: 'persistency' stamp: 'jmck 2/26/2011 23:09'!
reset
	List := nil.! !

!MODatalistObject class methodsFor: 'persistency' stamp: 'jmck 2/26/2011 23:22'!
setUpDatabase
	"override in your heirarchy root class to set PGConnectionArgs for your db"! !

!MODatalistObject class methodsFor: 'persistency' stamp: 'jmck 2/26/2011 23:09'!
shutDown
	self allSubclasses do: [ :each | each reset ]! !

!MODatalistObject class methodsFor: 'persistency' stamp: 'jmck 7/17/2012 17:27'!
startUp
	self shutDown.
	self setUpDatabase.
	self allSubclasses
		do: [:each | each isConcrete ifTrue: [ each list ]].
	Transcript cr; show: 'Moe startup complete'; cr! !

!MODatalistObject methodsFor: 'persistence' stamp: 'jmck 9/3/2011 23:22'!
delete
	super delete.
	self list remove: self ifAbsent: []! !

!MODatalistObject methodsFor: 'private-sql' stamp: 'jmck 2/26/2011 23:24'!
insert
	super insert.
	self list addLast: self.! !

!MODatalistObject methodsFor: 'persistence' stamp: 'jmck 6/9/2009 19:36'!
list
	^self class list! !

!MODatalistObject methodsFor: 'persistence' stamp: 'jmck 1/12/2013 13:18'!
loadEmbeddedObjectFor: aDescription withId: anId
	| cls |
	^ (cls := aDescription objectClass) list
		detect: [ :each | each id = anId ]
		ifNone: [  [ cls loadId: anId ] on: Error do: [ nil ]  ]! !

!MOObject class methodsFor: 'as yet unclassified' stamp: 'jmck 3/7/2011 20:56'!
descriptionContainer
	^ MOContainer new! !

!MOObject class methodsFor: 'as yet unclassified' stamp: 'jmck 1/18/2011 22:28'!
load: aRelation
	| coll | 
	coll := (super load: aRelation) asOrderedCollection.
	coll do: [ :each | each loadEmbedded ].
	^coll
	! !

!MOObject methodsFor: 'magritte' stamp: 'jmck 6/18/2009 21:08'!
attributeValueFor: aColumnName
	| desc value |
	desc := self description detect: [ :each | each sqlColumnName = aColumnName ] ifNone: [ ^nil ].
	((value := self readUsing: desc) isNil or: [
		(desc isEmbedded or: [ desc isEmbeddedCollection ]) not ])
		ifTrue: [ ^value ].
	^desc isEmbedded 
		ifTrue: [ value id ]
		ifFalse: [ value asIntegerArrayString ]
			! !

!MOObject methodsFor: 'persistence' stamp: 'jmck 2/10/2010 23:50'!
delete
	| |
	self description do: [:desc |
		desc isEmbeddedCollection
			ifTrue: [(self readUsing: desc) do: [ :child | child delete ]]].
	super delete.! !

!MOObject methodsFor: 'magritte' stamp: 'jmck 2/6/2010 20:14'!
hasEmbeddedChildren
	"return true if any embedded collection is not empty"
	| collection |
	self description
		do: [:desc | (desc isEmbeddedCollection
					and: [(collection := self readUsing: desc) notEmpty 
						and: [ collection anySatisfy: [ :each | each hasId ]]])
				ifTrue: [^ true]].
	^ false! !

!MOObject methodsFor: 'persistence' stamp: 'jmck 6/17/2009 06:44'!
insert
	| descriptions result value |
	descriptions := self mappedDescription children
		copyWithout: self class descriptionId.
	result := self relation basicQuery: (self relation
		sqlInsert: (descriptions collect: [ :each | each sqlColumnName  ])
		values: (descriptions collect: [ :each | 
			(((value := self readUsing: each) isNil) or: [
				(each isEmbedded or: [ each isEmbeddedCollection]) not ])
			ifTrue: [ value ]
			ifFalse: [
				each isEmbedded 
					ifTrue: [ value id ]
					ifFalse: [ value asIntegerArrayString ]]])) , ';' ,
		(self relation sqlCurrentValue: self class descriptionId sqlColumnName).
	id := result first first.
! !

!MOObject methodsFor: 'loading' stamp: 'jmck 6/11/2009 17:45'!
loadEmbedded
	self description do: [ :desc |
		desc isEmbedded 
			ifTrue: [ self loadEmbeddedDescription: desc ]
			ifFalse: [
				desc isEmbeddedCollection 
					ifTrue: [ 
						self loadEmbeddedCollectionDescription: desc ]]].
! !

!MOObject methodsFor: 'loading' stamp: 'jmck 5/27/2012 21:20'!
loadEmbeddedCollectionDescription: aDescription
	"Read the string representation of the integer array returned by postgres, convert it and retrieve the objects replacing the string with the ressurected collection"
	
	| ids embeddedObjects |
	embeddedObjects := OrderedCollection new.
	ids := self readUsing: aDescription.
	(ids notNil and: [ ids size > 2 ]) 
			ifTrue: [
				(ids asIntegerArray)
					inject: embeddedObjects
					into: [ :collection :i | 
						collection add: (self loadEmbeddedObjectFor: aDescription withId: i); yourself]].
	aDescription accessor write: embeddedObjects to: self ! !

!MOObject methodsFor: 'loading' stamp: 'jmck 6/13/2009 12:48'!
loadEmbeddedDescription: aDescription
	| embeddedid |
	(embeddedid := self readUsing: aDescription) isNil
		ifFalse: [
			aDescription accessor
				write: (self loadEmbeddedObjectFor: aDescription withId: embeddedid)
				to: self ]! !

!MOObject methodsFor: 'loading' stamp: 'jmck 6/11/2009 18:35'!
loadEmbeddedObjectFor: aDescription withId: anId
	"This can be overridden if you would rather hit the database for each embedded object
	The current implementation uses uses a class variable to store ALL objects from the DB"
	
	 self subclassResponsibility! !

!MOObject methodsFor: 'magritte' stamp: 'jmck 2/10/2010 06:18'!
mementoClass 
	^MOMemento ! !
MODatalistObject initialize!
